/*jslint plusplus:false,white:false,undef: false, rhino: true, onevar: true, evil: false */

/*
Title: i3Geo

Arquivo:

i3geo/classesjs/classe_i3geo.js

Licenca:

GPL2

I3Geo Interface Integrada de Ferramentas de Geoprocessamento para Internet

Direitos Autorais Reservados (c) 2006 Ministrio do Meio Ambiente Brasil
Desenvolvedor: Edmar Moretti edmar.moretti@mma.gov.br

Este programa  software livre; voc pode redistribu-lo
e/ou modific-lo sob os termos da Licena Pblica Geral
GNU conforme publicada pela Free Software Foundation;
tanto a verso 2 da Licena.
Este programa  distribudo na expectativa de que seja til,
porm, SEM NENHUMA GARANTIA; nem mesmo a garantia implcita
de COMERCIABILIDADE OU ADEQUAO A UMA FINALIDADE ESPECFICA.
Consulte a Licena Pblica Geral do GNU para mais detalhes.
Voc deve ter recebido uma cpia da Licena Pblica Geral do
GNU junto com este programa; se no, escreva para a
Free Software Foundation, Inc., no endereo
59 Temple Street, Suite 330, Boston, MA 02111-1307 USA.
*/
/*
Classe: i3GEO

A classe i3GEO possu os mtodos de criao e atualizao do mapa. Todas as subclasses
so baseadas em i3GEO, por exemplo, para criar uma janela flutuante sobre o mapa,
utilize i3GEO.janela.cria()

Para inicializar o mapa, utilize i3GEO.inicia() e para atualizar o mapa, utilize i3GEO.atualiza().
Aps terminado o processo de inicializao, pode-se executar uma funo de ajuste. Essa funo
deve ser definida em i3GEO.finaliza, por exemplo i3GEO.finaliza = "funcaoDeAjuste()"

Ao inicializar ou atualizar o i3Geo,  feita uma chamada em AJAX 
para a obteno dos parmetros necessrios ao funcionamento do mapa. Esses parmetros
so armazenados na varivel i3GEO.parametros

Nessa classe esto disponveis variveis internas utilizadas em vrias funes, como i3GEO.temaAtivo
*/
i3GEO = {
	
	/*
	Propriedade: parametros
	
	Parmetros obtidos do mapa atual. Os parmetros so fornecidos pelos programas
	PHP de redesenho e criao do mapa e atualizados sempre que o mapa  alterado.
	
	Exemplos:
	
	alert(i3GEO.parametros.mapexten);
	i3GEO.mapscale = 25000;
	
	Parametros:
	
	mapexten {String} - extenso geogrfica do mapa no formato xmin ymin xmax ymax
	
	mapscale {Numeric} - denominador da escala do mapa
	
	mapres {Numeric} - resoluo da imagem do mapa em DPI
	
	pixelsize {Numeric} - tamanho em unidades de terreno dos pixels da imagem
	
	mapfile {String} - nome do mapfile atualmente em uso
	
	cgi {String} - endereo do executvel do mapserver no servidor acessvel pela URL
	
	extentTotal {String} - extenso do mapa na inicializao
	
	mapimagem {String} - URL da imagem que compe o mapa
	
	geoip {sim|nao} - indica se o geoip est instalado
	
	listavisual {String} - lista de visuais disponveis
	
	utilizacgi {sim|nao} - indica se o mapa atual est no modo CGI
	
	versaoms {String} - verso do Mapserver instalado no servidor (maior)
	
	versaomscompleta {String} - verso do Mapserver instalado no servidor (completa)
	
	mensagens {String} - mensagens para uso no letreiro
	
	w {Integer} - largura do mapa atual
	
	h {Integer} - altura do mapa atual
	
	locsistemas {String} - endereo do xml com a lista de sistemas adicionais
	
	locidentifica {String} - endereo do xml com a lista de sistemas de identificao
	
	r {sim|nao} - indica se o software R est instalado no servidor
	
	locmapas {String} - endereo do xml com a lista de mapas
	
	extentref {String} - extenso geogrfica do mapa de referncia
	
	celularef {Numeric} - tamanho do pixel do mapa de referncia em unidades do terreno
	
	kmlurl {String} - url de um arquivo kml que ser inserido no mapa. Vlido apenas na interface Google Maps

	mensagemInicia {String} - mensagem definida em ms_configura.php indicando a verso atual do i3Geo.
	*/
	parametros: {
		mapexten: "",
		mapscale: "",
		mapres: "",
		pixelsize: "",
		mapfile: "",
		cgi: "",
		extentTotal: "",
		mapimagem: "",
		geoip: "",
		listavisual: "",
		utilizacgi:"",
		versaoms:"",
		versaomscompleta: "",
		mensagens:"",
		w: "",
		h: "",
		locsistemas:"",
		locidentifica:"",
		r:"",
		locmapas:"",
		celularef:"",
		kmlurl:"",
		mensageminicia:""
	},
	/*
	Propriedade: finaliza
	
	Armazena o nome de uma funo que ser executada aps a inicializao do mapa
	
	Tipo:
	{string}
	
	Default:
	{""}
	*/
	finaliza:"",
	/*
	Variavel: temaAtivo
	
	Indica o ltimo tema que foi ativado no mapa
	
	Permite que ao se passar de uma ferramenta
	para outra, os menus reflitam a ltima escolha.
	
	Tipo:
	{string}
	
	Default:
	{""}
	*/
	temaAtivo: "",
	/*
	Variavel: contadorAtualiza
	
	Essa varivel  um contador utilizado para indicar quantos processos esto ativos e que
	faro o redesenho do mapa. O mapa s  atualizado quando o contador for menor que 1.
	Essi contador  utilizado no mtodo i3GEO.atualiza e todas as funes que chamam esse
	mtodo devem acrescentar 1 ao contador, por exemplo, i3GEO.contadoratualiza++
	O contador  necessrio para evitar chamadas desnecessrias  funo de redesenho do mapa.
	
	Tipo:
	{string}
	*/
	contadorAtualiza:0,
	/*
	Function: cria
	
	Cria e configura a visualizao do mapa, definindo posicionamentos, tamanho etc
	
	Aps as definies bsicas,  executado o programa <i3GEO.Interface.cria> que ir
	realizar as operaes conforme a interface atualmente em uso. A interface  definida
	em <i3GEO.Interface.ATUAL>
	
	Veja <i3GEO.Interface>
	*/
	cria:function(){
		if(typeof(console) !== 'undefined'){console.info("i3GEO.cria()");}
		var temp,i,tamanho;
		if (window.location.href.split("?")[1]){
			i3GEO.configura.sid = window.location.href.split("?")[1];
			//
			//a biblioteca YUI, por algum motivo, acrescenta # na URL. O # precisa ser removido, caso contrrio, a opo de reload da pgina pelo browser as vezes no funciona
			//
			if (i3GEO.configura.sid.split("#")[0])
			{i3GEO.configura.sid = i3GEO.configura.sid.split("#")[0];}
		}
		else
		{i3GEO.configura.sid = "";}
		//
		//para efeitos de compatibilidade
		//
		g_panM = "nao";
		g_sid = i3GEO.configura.sid;
		try {i3GEO.configura.locaplic = g_locaplic;}
		catch(e){
			g_locaplic = i3GEO.configura.locaplic;
		}
		try{i3GEO.configura.diminuixM = g_diminuixM;}catch(e){}
		try{i3GEO.configura.diminuixN = g_diminuixN;}catch(e){}
		try{i3GEO.configura.diminuiyM = g_diminuiyM;}catch(e){}
		try{i3GEO.configura.diminuiyN = g_diminuiyN;}catch(e){}			
		//
		//calcula o tamanho do mapa
		//
		tamanho = i3GEO.calculaTamanho();
		i3GEO.Interface.cria(tamanho[0],tamanho[1]);
		i3GEO.parametros = {
			mapexten: "",
			mapscale: "",
			mapres: "",
			pixelsize: "",
			mapfile: "",
			cgi: "",
			extentTotal: "",
			mapimagem: "",
			geoip: "",
			listavisual: "",
			utilizacgi:"",
			versaoms:"",
			versaomscompleta:"",
			mensagens:"",
			w: tamanho[0],
			h: tamanho[1],
			locsistemas:"",
			locidentifica:"",
			r:"",
			locmapas:"",
			extentref:"",
			kmlurl:"",
			mensageminicia:""
		};
		if(tamanho[0] < 550){
			i = $i(i3GEO.gadgets.PARAMETROS.mostraQuadros.idhtml);
			if(i){i.style.display = "none";}
		}
	},
	/*
	Function: inicia
	
	Inicializa o mapa
	
	Verifica se o mapa apresenta algum problema no processamento no lado do servidor e
	realiza as operaes de tentativa de recuperao, se for o caso
	
	No incio do processo  executada a funo <i3GEOmantemCompatibilidade>
	para realizar as operaes necessrias de manuteno de compatibilidade da verso atual para as anteriores
	
	A inicializao  baseada em <i3GEO.php.inicia> cujo retorno  utilizado para definir a
	varivel <i3GEO.parametros>
	
	Aps a inicializao  executado <i3GEO.Interface.inicia>
	*/
	inicia:function(){
		if(typeof(console) !== 'undefined'){console.info("i3GEO.inicia()");}
		var montaMapa,mashup,tamanho;
		if(typeof("i3GEOmantemCompatibilidade") === 'function')
		{i3GEOmantemCompatibilidade();}
		montaMapa = function(retorno){
			var tempo,titulo,temp,abreJM;
			if(retorno === ""){
				alert("Ocorreu um erro no mapa - montaMapa");
				retorno = {data:{erro: "erro"}};
			}
			if(retorno.data.erro){
				i3GEO.janela.fechaAguarde("montaMapa");
				document.body.style.backgroundColor="white";
				document.body.innerHTML = "<br>Para abrir o i3Geo utilize o link:<br><a href="+i3GEO.configura.locaplic+"/ms_criamapa.php >"+i3GEO.configura.locaplic+"/ms_criamapa.php</a>";
				return("linkquebrado");
			}
			else{
				if(retorno.data.variaveis){
					//
					//executa com eval a string que  retornada pelo servidor (funo inicia do mapa_controle.php
					//
					tempo = "";
					titulo = "";
					eval(retorno.data.variaveis);
					try{
						if (titulo !== "")
						{top.document.title = titulo;}
					}
					catch(e){
						if(typeof(console) !== 'undefined'){console.error(e);}
					}
					i3GEO.ajuda.mostraJanela("Tempo de desenho em segundos: "+tempo,"");
					try{
						i3GEO.parametros.mapexten= mapexten;
						i3GEO.parametros.mapscale= parseInt(mapscale,10);
						i3GEO.parametros.mapres= mapres;
						i3GEO.parametros.pixelsize= g_celula;
						i3GEO.parametros.mapfile= mapfile;
						i3GEO.parametros.cgi= cgi;
						i3GEO.parametros.extentTotal=mapexten;
						i3GEO.parametros.mapimagem= mapimagem;
						i3GEO.parametros.geoip= geoip;
						i3GEO.parametros.listavisual= listavisual;
						i3GEO.parametros.utilizacgi= utilizacgi;
						i3GEO.parametros.versaoms= versaoms;
						i3GEO.parametros.mensagens= mensagens;
						i3GEO.parametros.locsistemas = locsistemas;
						i3GEO.parametros.locidentifica = locidentifica;
						i3GEO.parametros.r = r;
						i3GEO.parametros.locmapas = locmapas;
						i3GEO.parametros.extentref = extentref;
						i3GEO.parametros.versaoms = versaoms;
						i3GEO.parametros.versaomscompleta = versaomscompleta;
						i3GEO.parametros.kmlurl = kmlurl;
						i3GEO.parametros.mensageminicia = mensagemInicia;
					}
					catch(e){alert("Erro durante a definicao de i3GEO.parametros "+e);}					
					i3GEO.gadgets.quadros.inicia(10);
					i3GEO.gadgets.quadros.grava("extensao",mapexten);
					
					i3GEO.arvoreDeCamadas.cria("",retorno.data.temas,i3GEO.configura.sid,i3GEO.configura.locaplic);
					i3GEO.util.arvore("<b>"+$trad("p13")+"</b>","listaPropriedades",i3GEO.configura.listaDePropriedadesDoMapa);

					i3GEO.gadgets.mostraBuscaRapida();
					i3GEO.gadgets.mostraVersao();
					i3GEO.guias.cria();
					if($i("arvoreAdicionaTema"))
					{i3GEO.arvoreDeTemas.cria(i3GEO.configura.sid,i3GEO.configura.locaplic,"arvoreAdicionaTema");}
					
					if($i("mst")){$i("mst").style.display="block";}
					i3GEO.atualiza(retorno);
					//
					//calcula (opcional) o tamanho correto da tabela onde fica o mapa
					//se no for feito esse clculo, o mapa fica ajustado  esquerda
					//			
					temp = 0;
					if ($i("contemFerramentas")){temp = temp + parseInt($i("contemFerramentas").style.width,10);}
					if ($i("ferramentas")){temp = temp + parseInt($i("ferramentas").style.width,10);}
					if($i("mst"))
					{$i("mst").style.width=i3GEO.parametros.w + temp + "px";}
					if (i3GEO.configura.entorno === "sim"){
						i3GEO.configura.entorno = "nao";
						i3GEO.navega.entorno.ativaDesativa();
					}
					i3GEO.navega.autoRedesenho.ativa();
					if ($i("i3geo_escalanum")){$i("i3geo_escalanum").value = i3GEO.parametros.mapscale;}
					if ((i3GEO.parametros.geoip === "nao") && ($i("ondeestou")))
					{$i("ondeestou").style.display="none";}
					i3GEO.Interface.inicia();
				}
				else
				{alert("Erro. Impossivel criar o mapa "+retorno.data);return;}
				//
				//ativa a janela de mensagens se for o caso
				//
				if(document.getElementById("ajuda")) //para efeitos de compatibilidade com as verses anteriores a 4.1
				{i3GEO.ajuda.DIVAJUDA = "ajuda";}
				abreJM = "sim";
				if(i3GEO.util.pegaCookie("g_janelaMen")){
					abreJM = i3GEO.util.pegaCookie("g_janelaMen");
					i3GEO.configura.iniciaJanelaMensagens = (abreJM === "sim") ? true : false;
				}
				if(i3GEO.configura.iniciaJanelaMensagens === true)
				{i3GEO.ajuda.abreJanela();}		
				i3GEO.janela.fechaAguarde("montaMapa");
				if (i3GEO.configura.liberaGuias === "sim")
				{i3GEO.guias.libera();}
			}
			if($i("mst")){$i("mst").style.visibility ="visible";}	
		};
		if (!$i("i3geo"))
		{document.body.id = "i3geo";}
		$i("i3geo").className = "yui-skin-sam";
		//if($i("mst"))
		//{$i("mst").style.visibility ="hidden";}
		//
		//se i3GEO.configura.sid = "", o html foi aberto diretamente
		//ento,  necessrio criar os arquivos temporrios do mapa
		//essa operao deve ser assncrona
		//
		if(i3GEO.configura.sid===""){
			mashup = function (retorno){
				i3GEO.configura.sid = retorno.data;
				i3GEO.inicia();
			};
			i3GEO.php.criamapa(mashup,i3GEO.configura.mashuppar);
		}
		else{
			//YAHOO.log("Chamada AJAX para obter o mapa inicial", "i3geo");
			i3GEO.janela.abreAguarde("montaMapa",$trad("o5"));
			if(i3GEO.parametros.w === "" || i3GEO.parametros.h === ""){
				tamanho = i3GEO.calculaTamanho();
				i3GEO.parametros.w = tamanho[0];
				i3GEO.parametros.h = tamanho[1];
			}
			i3GEO.php.inicia(montaMapa,i3GEO.configura.embedLegenda,i3GEO.parametros.w,i3GEO.parametros.h);
		}
		if(i3GEO.eventos.NAVEGAMAPA.toString().search("i3GEO.janela.fechaAguarde()") < 0)
		{i3GEO.eventos.NAVEGAMAPA.push("i3GEO.janela.fechaAguarde()");}
		eval(i3GEO.finaliza);
	},
	/*
	Function: atualiza
	
	Atualiza o mapa atual, altera a imagem do mapa os gadgets ativos e os parmetros e
	verifica a integridade do mapa em uso (arquivo mapfile)
	
	O processo executa tambm a funo de atualizao especfica da interface atual em uso, veja
	<i3GEO.Interface.redesenha>
	
	Os seguintes gadgets so processados
	
	<i3GEO.arvoreDeCamadas.atualiza>
	
	<i3GEO.arvoreDeCamadas.atualizaFarol>
	
	Os eventos definidos em <i3GEO.eventos.navegaMapa> so executados
	
	Parametro:
	
	retorno {String} - string com os parmetros do novo mapa. Se retorno no
	for especificado ou se for vazio, ser feita uma chamada em ajax para sua obteno. O resultado
	dessa chamada  armazenada em i3GEO.parametros
	*/
	atualiza: function(retorno){
		if(typeof(console) !== 'undefined'){console.info("i3GEO.atualiza()");}
		var corpoMapa,erro,tempo,mapscale,mapexten;
		if(i3GEO.contadorAtualiza > 1)
		{i3GEO.contadorAtualiza--;return;}
		if(i3GEO.contadorAtualiza > 0)
		{i3GEO.contadorAtualiza--;}
		corpoMapa = function(){
			if($i("ajaxCorpoMapa"))
			{return;}
			i3GEO.janela.abreAguarde("ajaxCorpoMapa",$trad("o1")+" atualizando");
			i3GEO.php.corpo(i3GEO.atualiza,i3GEO.configura.tipoimagem);		
		};
		if(arguments.length === 0){
			i3GEO.janela.fechaAguarde("ajaxCorpoMapa");
			corpoMapa.call();
			return;
		}
		if(retorno === ""){
			corpoMapa.call();
			return;
		}
		if(!retorno.data){
			alert(retorno);
			i3GEO.mapa.recupera.inicia();
			//corpoMapa.call();
			return;
		}
		//verifica se o parmetro retorno existe, caso contrrio,
		//faz a chamada ao programa PHP para obter os parmetros
		try{
			if (retorno.data === "erro"){
				alert("Erro no mapa. Sera feita uma tentativa de recuperacao.");
				i3GEO.mapa.recupera.inicia();return;
			}
			else
			if(retorno.data === "ok" || retorno.data === "")
			{corpoMapa.call();return;}
		}
		catch(e){
			if(typeof(console) !== 'undefined'){console.error(e);}
		}
		erro = function(){
			var legimagem,c;
			legimagem = "";
			c = confirm("Ocorreu um erro, quer tentar novamente?");
			if(c){
				corpoMapa.call();
			}
			else{
				i3GEO.janela.fechaAguarde();
			}
			return;
		};
		try{eval(retorno.data.variaveis);}
		catch(e){
			if(typeof(console) !== 'undefined'){console.error(e);}
			erro.call();
			return;
		}
		if(arguments.length === 0 || retorno === "" || retorno.data.variaveis === undefined){erro.call();return;}
		else{	
			if(arguments.length === 0){return;}
			i3GEO.mapa.verifica(retorno);
			tempo = "";
			if(i3GEO.desenho.richdraw)
			{i3GEO.desenho.richdraw.clearWorkspace();}
			mapscale = "";
			mapexten = "";
			//transforma o retorno em variveis
			eval(retorno.data.variaveis);
			//
			//o try aqui  necessrio pois na interface googlemaps os parmetros retorno.data.variaveis no so gerados completamente
			//
			try{
				i3GEO.arvoreDeCamadas.atualiza(retorno.data.temas);
				if (i3GEO.parametros.mapscale !== mapscale)
				{i3GEO.arvoreDeCamadas.atualizaFarol(mapscale);}
				i3GEO.parametros.mapexten = mapexten;
				i3GEO.parametros.mapscale = mapscale;
				i3GEO.parametros.mapres = mapres;
				i3GEO.parametros.pixelsize = g_celula;
				i3GEO.parametros.mapimagem = mapimagem;
			}
			catch(e){
				if(typeof(console) !== 'undefined'){console.error(e);}
			}
			i3GEO.Interface.redesenha();
			//caso esteja na funo de identificao
			if($i("i3GEOidentificalistaTemas")){
				g_tipoacao = "identifica";
				g_operacao='identifica';
			}
			else
			{g_operacao = "";}
			i3GEO.parametros.mapexten = mapexten;
			if ($i("mensagemt"))
			{$i("mensagemt").value = i3GEO.parametros.mapexten;}
			
			i3GEO.arvoreDeCamadas.CAMADAS = retorno.data.temas;
			i3GEO.eventos.navegaMapa();
			if (i3GEO.configura.entorno === "sim"){
				i3GEO.navega.entorno.geraURL();
				i3GEO.navega.entorno.ajustaPosicao();
			}
			i3GEO.ajuda.mostraJanela("Tempo de redesenho em segundos: "+tempo,"");
		}	
	},
	/*
	Function: calculaTamanho
	
	Calcula o tamanho do mapa atual e define alguns elementos HTML do mapa
	
	Return:
	{array} - [w,h]
	*/
	calculaTamanho: function(){
		if(typeof(console) !== 'undefined'){console.info("i3GEO.calculaTamanho()");}
		var diminuix,diminuiy,menos,novow,novoh,w,h,temp;
		diminuix = (navm) ? i3GEO.configura.diminuixM : i3GEO.configura.diminuixN;
		diminuiy = (navm) ? i3GEO.configura.diminuiyM : i3GEO.configura.diminuiyN;
		//
		//subtrai barra de rolagem
		//
		try{diminuiy += i3GEO.util.getScrollerWidth();}
		catch(e){
			if(typeof(console) !== 'undefined'){console.error(e);}
		}
		//
		menos = 0;
		if ($i("contemFerramentas"))
		{menos += parseInt($i("contemFerramentas").style.width,10);}
		if ($i("ferramentas"))
		{menos += parseInt($i("ferramentas").style.width,10);}
		novow = parseInt(screen.availWidth,10) - diminuix;
		novoh = parseInt(screen.availHeight,10) - diminuiy;
		if (window.top === window.self){//nao se aplica em iframe		
			window.resizeTo(screen.availWidth,screen.availHeight);
			window.moveTo(0,0);
		}
		//o try aqui  necessrio por conta do uso possvel do i3geo em um iframe
		try{
			if (novow < 800){
				novow = 800;
				novoh = 600;
			}
		}
		catch(e){
			if(typeof(console) !== 'undefined'){console.error(e);}
		}
		//novoh = 500
		document.body.style.width = novow - diminuix;
		document.body.style.height = novoh;
		w = novow - menos - diminuix;
		h = novoh - diminuiy;
		temp = $i("corpoMapa");
		if (temp){
			if(temp.style){
				if (temp.style.width){
					w = parseInt(temp.style.width,10);
					h = parseInt(temp.style.width,10);
				}
				if (temp.style.height)
				{h = parseInt(temp.style.height,10);}
			}
		}
		temp = $i("contemImg");
		if(temp){
			temp.style.height=h + "px";
			temp.style.width=w + "px";
		}
		return [w,h];	
	}
};
/*
Classe: i3GEOF

Esta classe recebe os mtodos sob demanda por meio da injeo de javascripts por meio de script tag.
*/
i3GEOF = [];
//YAHOO.log("carregou classe i3geo", "Classes i3geo");